
package com.bweb.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.pay.util.sdk.WXPayConstants;
import com.hai.common.utils.HttpsUtils;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.common.utils.WxUtils;
import com.hai.config.HuiLianTongUnionCardConfig;
import com.hai.entity.*;
import com.hai.model.ResponseData;
import com.hai.openApi.config.BlxConfig;
import com.hai.order.service.OrderService;
import com.hai.order.type.OrderRefundOpUserType;
import com.hai.order.utils.OrderUtil;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.bouncycastle.LICENSE;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/26 23:08
 */
@Controller
@RequestMapping(value = "/test")
@Api(value = "订单接口")
public class HighTestController {

    private static Logger log = LoggerFactory.getLogger(HighTestController.class);



    @Resource
    private OrderService orderService;
    @Resource
    private HighUserCardService highUserCardService;

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private BlxConfig blxConfig;

    @RequestMapping(value="/resolveResponse",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "解析")
    public ResponseData resolveResponse(
            @RequestParam(name = "data", required = false) String data
    ) {
        try {
            JSONObject cardInfoObject = HuiLianTongUnionCardConfig.resolveResponse(data);
            return ResponseMsgUtil.success(cardInfoObject);

        } catch (Exception e) {
            log.error("getUserByTelephone",e);
            return ResponseMsgUtil.exception(e);
        }
    }



    @RequestMapping(value = "/getHuiLianTongCardConsume", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询汇联通工会卡消费记录")
    public ResponseData getHuiLianTongCardConsume(@RequestParam(name = "businessType", required = true) String businessType,
                                                  @RequestParam(name = "cardNo", required = true) String cardNo,
                                                  @RequestParam(name = "sdate", required = true) Long sdate,
                                                  @RequestParam(name = "edate", required = true) Long edate,
                                                  @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                                  @RequestParam(name = "pageSize", required = true) Integer pageSize,
                                                  HttpServletRequest request) {
        try {


            JSONObject consumptionRecord = HuiLianTongUnionCardConfig.queryConsumptionRecordByBusiness(businessType, cardNo, sdate, edate, pageNum, pageSize);
            if (StringUtils.isBlank(consumptionRecord.getString("data"))) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQUEST_ERROR, "");
            }

            JSONArray array = HuiLianTongUnionCardConfig.resolveResponse(consumptionRecord.getString("data")).getJSONArray("records");

            return ResponseMsgUtil.success(array);

        } catch (Exception e) {
            log.error("HighUserCardController --> getHuiLianTongCardConsume() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }



//    @RequestMapping(value = "/createOrder", method = RequestMethod.POST)
//    @ResponseBody
//    @ApiOperation(value = "创建订单")
//    public ResponseData createOrder(@RequestBody JSONObject object) {
//        try {
//
//            Map<String , Object> mapPost = new HashMap<>();
//            mapPost.put("apiKey" , "0C7897EA3BD96156A5881618FF2FF645");
//            mapPost.put("merchId" , "20230258302");
//            mapPost.put("timetable" , new Date().getTime());
//            mapPost.put("orderNo" , "TEST" +  OrderUtil.generateOrderNo());
//            mapPost.put("productType" , object.getString("productType"));
//            mapPost.put("notifyUrl" , "test");
//
//            mapPost.put("content" , object.toString());
//            System.out.println(mapPost);
//
//            Map<String , Object> map = new HashMap<>();
//            map.put("apiKey" , "0C7897EA3BD96156A5881618FF2FF645");
//            map.put("merchId" , "20230258302");
//            map.put("timetable" , mapPost.get("timetable").toString());
//            map.put("orderNo" , mapPost.get("orderNo"));
//            map.put("productType" , object.getString("productType"));
//            map.put("content" , object.toString());
//
//            System.out.println(map);
//
//            mapPost.put("sign" , WxUtils.generateSignApi(map, MapUtils.getString(map, "apiKey"), WXPayConstants.SignType.MD5));
//            return ResponseMsgUtil.success(HttpsUtils.doPost("http://localhost:9902/v1/order/createOrder" , mapPost , new HashMap<>()));
//        } catch (Exception e) {
//            log.error("HighOrderController --> getBackendToken() error!", e);
//            return ResponseMsgUtil.exception(e);
//        }
//    }

    @RequestMapping(value = "/payOrder", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "支付订单")
    public ResponseData payOrder(@RequestParam(name = "mchOrderNo", required = true) String mchOrderNo) {
        try {

            Map<String , Object> mapPost = new HashMap<>();
            mapPost.put("apiKey" , "0C7897EA3BD96156A5881618FF2FF645");
            mapPost.put("merchId" , "20230258302");
            mapPost.put("mchOrderNo" , mchOrderNo);


            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , "0C7897EA3BD96156A5881618FF2FF645");
            map.put("merchId" , "20230258302");
            map.put("mchOrderNo" , mchOrderNo);

            System.out.println(map);

            mapPost.put("sign" , WxUtils.generateSignApi(map, MapUtils.getString(map, "apiKey"), WXPayConstants.SignType.MD5));
            return ResponseMsgUtil.success(HttpsUtils.doPost("http://localhost:9902/v1/order/orderPay" , mapPost , new HashMap<>()));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/queryOrderDetail", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询订单详情")
    public ResponseData queryOrderDetail(@RequestParam(name = "mchOrderNo", required = true) String mchOrderNo) {
        try {

            Map<String , Object> mapPost = new HashMap<>();
            mapPost.put("apiKey" , "0C7897EA3BD96156A5881618FF2FF645");
            mapPost.put("merchId" , "20230258302");
            mapPost.put("mchOrderNo" , mchOrderNo);


            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , "0C7897EA3BD96156A5881618FF2FF645");
            map.put("merchId" , "20230258302");
            map.put("mchOrderNo" , mchOrderNo);

            System.out.println(map);

            mapPost.put("sign" , WxUtils.generateSignApi(map, MapUtils.getString(map, "apiKey"), WXPayConstants.SignType.MD5));
            return ResponseMsgUtil.success(HttpsUtils.doPost("http://localhost:9902/v1/order/queryOrderDetail" , mapPost , new HashMap<>()));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/deposit", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "工会卡充值")
    public ResponseData deposit(@RequestParam(name = "orderNo", required = true) String orderNo) {
        try {

            HighOrder order = highOrderService.getOrderByOrderNo(orderNo);
            HighChildOrder highChildOrder = order.getHighChildOrderList().get(0);

            if (order.getOrderStatus() != 2) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "订单状态错误");
            }

            // 汇联通充值
            String goodsDesc = "汇联通充值"+order.getPayPrice()+"元";
            String tranDesc = "";
            String businessType = "ghk_deposit";

            String orderNoAgain = OrderUtil.generateOrderNo();

            order.setOrderStatus(Integer.valueOf(3));
            order.setRemarks("工会卡再次充值订单号" + orderNoAgain);

            highOrderService.updateOrder(order);

            // 汇联通卡充值
            JSONObject deposit = HuiLianTongUnionCardConfig.deposit(orderNoAgain, highChildOrder.getGoodsSpecName(), order.getPayPrice(), businessType, order.getPaySerialNo(), goodsDesc);

            return ResponseMsgUtil.success( HuiLianTongUnionCardConfig.resolveResponse(deposit.getString("data")));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/getHuiLianTongCardByCardNo", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据卡号查询汇联通工会卡详情")
    public ResponseData getHuiLianTongCardByCardNo(@RequestParam(name = "cardNo", required = true) String cardNo) {
        try {
            // 查询工会卡
            JSONObject cardInfo = HuiLianTongUnionCardConfig.queryBalance(cardNo);
            JSONObject resolveResponse = HuiLianTongUnionCardConfig.resolveResponse(cardInfo.getString("data"));
            return ResponseMsgUtil.success(resolveResponse);

        } catch (Exception e) {
            log.error("HighUserCardController --> getHuiLianTongCardByCardNo() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/consumption", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "工会卡支付")
    public ResponseData consumption(@RequestParam(name = "payRealPrice", required = true) BigDecimal payRealPrice,
                                    @RequestParam(name = "cardNo", required = true) String cardNo) {
        try {


            String goodsDesc = "";
            String tranDesc = "";
            String instCode = "11101527";
            String businessType = "hisen_consume";


            // 工会卡支付
            JSONObject consumption = HuiLianTongUnionCardConfig.consumption(OrderUtil.generateOrderNo(), cardNo, payRealPrice, businessType, instCode, goodsDesc, tranDesc);
            if (!consumption.getString("respCode").equals("0000")) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, consumption.getString("respMessage"));
            }
            // 响应参数
            JSONObject consumptionResult = HuiLianTongUnionCardConfig.resolveResponse(consumption.getString("data"));
            if (consumptionResult.getBoolean("success") != true) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, consumptionResult.getString("message"));
            }
            return ResponseMsgUtil.success(consumptionResult);

        } catch (Exception e) {
            log.error("HighUserCardController --> getHuiLianTongCardByCardNo() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/refundOrder", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "工会卡退款")
    public ResponseData refundOrder(@RequestParam(name = "orderNo", required = true) String orderNo) {
        try {

            HighOrder order = highOrderService.getOrderByOrderNo(orderNo);

            if (order.getOrderStatus() != 2) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "订单状态错误");
            }

            orderService.refundOrder(order, "工会卡充值退款",OrderRefundOpUserType.TYPE1,
                    null,
                    "系统自动退款");

            return ResponseMsgUtil.success("退款成功");
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/externalOrderRefund", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "对外订单退款")
    public ResponseData externalOrderRefund(@RequestParam(name = "paySerialNo", required = true) String paySerialNo,
                                            @RequestParam(name = "WxMchId", required = true) String WxMchId,
                                            @RequestParam(name = "payRealPrice", required = true) String payRealPrice) {
        try {


            Map<String , Object> mapPost = new HashMap<>();
            mapPost.put("apiKey" , "B94B5E62807FA56A7260AB581D8A1B48");
            mapPost.put("merchId" , "20230394901");
            mapPost.put("paySerialNo" , paySerialNo);
            mapPost.put("wxMchId" , WxMchId);
            mapPost.put("payRealPrice" , payRealPrice);
            mapPost.put("refundPrice" , payRealPrice);

            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , "B94B5E62807FA56A7260AB581D8A1B48");
            map.put("merchId" , "20230394901");
            map.put("paySerialNo" , paySerialNo);
            map.put("wxMchId" , WxMchId);
            map.put("payRealPrice" , payRealPrice);
            map.put("refundPrice" , payRealPrice);

            System.out.println(map);

            mapPost.put("sign" , WxUtils.generateSignApi(map, MapUtils.getString(map, "apiKey"), WXPayConstants.SignType.MD5));
            return ResponseMsgUtil.success(HttpsUtils.doPost("https://hsg.dctpay.com/v1/order/externalOrderRefund" , mapPost , new HashMap<>()));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getBlxCouponBalance", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询停车券")
    public ResponseData getBlxCouponBalance(@RequestParam(name = "phone", required = true) String phone) {
        try {


            highUserCardService.getBlxCouponBalance(phone);

            return ResponseMsgUtil.success("object");
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/queryJsParking", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询捷顺停车场")
    public ResponseData queryJsParking(@RequestParam(name = "gps", required = true) String gps) {
        try {

            Map<String , Object> mapPost = new HashMap<>();
            mapPost.put("merchId" , "20230258302");
            mapPost.put("gps" , gps);
            mapPost.put("range" , "1000");


            mapPost.put("sign" , WxUtils.generateSignApi(mapPost, "0C7897EA3BD96156A5881618FF2FF645", WXPayConstants.SignType.MD5));
            return ResponseMsgUtil.success(HttpsUtils.doPost("http://localhost:9902/v1/blxCoupon/queryJsParking" , mapPost , new HashMap<>()));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getUserCouponPage", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询捷顺停车场")
    public ResponseData getUserCouponPage(@RequestParam(name = "phone", required = true) String phone) {
        try {

            Map<String , Object> mapPost = new HashMap<>();
            mapPost.put("merchId" , "20230258302");
            mapPost.put("phone" , phone);


            mapPost.put("sign" , WxUtils.generateSignApi(mapPost, "0C7897EA3BD96156A5881618FF2FF645", WXPayConstants.SignType.MD5));
            return ResponseMsgUtil.success(HttpsUtils.doPost("http://localhost:9902/v1/blxCoupon/getUserCouponPage" , mapPost , new HashMap<>()));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/queryJsOrderList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询车牌订单列表信息")
    public ResponseData queryJsOrderList(@RequestParam(name = "carNo", required = true) String carNo,
                                         @RequestParam(name = "orderStatus", required = true) String orderStatus) {
        try {

            Map<String , Object> mapPost = new HashMap<>();
            mapPost.put("merchId" , "20230258302");
            mapPost.put("orderStatus" , orderStatus);
            mapPost.put("carNo" , carNo);


            mapPost.put("sign" , WxUtils.generateSignApi(mapPost, "0C7897EA3BD96156A5881618FF2FF645", WXPayConstants.SignType.MD5));
            return ResponseMsgUtil.success(HttpsUtils.doPost("http://localhost:9902/v1/blxCoupon/queryJsOrderList" , mapPost , new HashMap<>()));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getCouponList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询车牌订单列表信息")
    public ResponseData getCouponList() {
        try {

            Map<String , Object> mapPost = new HashMap<>();
            mapPost.put("merchId" , "20230258302");



            mapPost.put("sign" , WxUtils.generateSignApi(mapPost, "0C7897EA3BD96156A5881618FF2FF645", WXPayConstants.SignType.MD5));
            return ResponseMsgUtil.success(HttpsUtils.doPost("http://localhost:9902/v1/blxCoupon/getCouponList" , mapPost , new HashMap<>()));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/sendCoupon", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "发放券")
    public ResponseData sendCoupon(@RequestParam(name = "couponCode", required = true) String couponCode,
                                   @RequestParam(name = "receiverCount", required = true) String receiverCount,
                                      @RequestParam(name = "receiverPhone", required = true) String receiverPhone) {
        try {

            Map<String , Object> mapPost = new HashMap<>();
            mapPost.put("merchId" , "20230258302");



            mapPost.put("sign" , WxUtils.generateSignApi(mapPost, "0C7897EA3BD96156A5881618FF2FF645", WXPayConstants.SignType.MD5));
            return ResponseMsgUtil.success(HttpsUtils.doPost("http://localhost:9902/v1/blxCoupon/getCouponList" , mapPost , new HashMap<>()));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

}


